/* Lanczos2 sRGB Resampler v0.1
 * By DOLLS
 * Do not distribute a modified version without permission. 
 */

texture tex1, tex2;
float2 rcpres;
sampler s0 = sampler_state {
	texture = <tex1>;
	AddressU = mirror;
	AddressV = mirror;
};

sampler s1 = sampler_state {
	texture = <tex2>;
	AddressU = mirror;
	AddressV = mirror;
};

// Linear RGB to sRGB and viceversa.
#define	GAMMA		2.35
#define	INV_GAMMA	1.0 / GAMMA

float4 Linear(float4 color) {
	return float4((color.r <= 0.0031308)? 12.92 * color.r : pow(1.055 * color.r, INV_GAMMA) - 0.055, (color.g <= 0.0031308)? 12.92 * color.g : pow(1.055 * color.g, INV_GAMMA) - 0.055, (color.b <= 0.0031308)? 12.92 * color.b : pow(1.055 * color.b, INV_GAMMA) - 0.055, color.a);
}

float4 Nonlinear(float4 color) {
	return float4((color.r <= 0.04045)? color.r / 12.92 : pow((color.r + 0.055) / 1.055, GAMMA), (color.g <= 0.04045)? color.r / 12.92 : pow((color.g + 0.055) / 1.055, GAMMA), (color.b <= 0.04045)? color.b / 12.92 : pow((color.b + 0.055) / 1.055, GAMMA), color.a);	
}

int2 res;

float4 Normalize(in float2 coord : TEXCOORD0): COLOR0
{		
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 loc = coord * float2(rcpres.y * res.y / (rcpres.x * res.x), 1.0) / (res.y * rcpres.y);
	return Nonlinear(tex2D(s0, loc));
}

#define	PI	3.1415926535897932384626433832795

float getWeight(float x)
{	
	if (x == 0.0) return 1.0;
    else if (x <= -2.0 || x >= 2.0) return 0.0;
    else {
		float tmp = x * PI;
		return 2.0 * sin(tmp) * sin(tmp / 2.0) / (tmp * tmp);
	}	
}

float4 Resample(in float2 coord : TEXCOORD0) : COLOR0
{	
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 pos = coord * res - 0.5;
	float2 tex = (floor(pos) + 0.5) * rcpres;
	
	float2 delta = frac(pos);	
	float weight[4];
	float contrib = 0.0;
	
	for (int i = -1; i <= 2; i++)
		contrib += weight[i + 1] = getWeight(abs(i - delta.y));
		
	float4 value = float4(0.0, 0.0, 0.0, 0.0);
	for (int i = -1; i <= 2; i++) {                       
		float ele = getWeight(abs(delta.x - i));		
		float j = i * rcpres.x;
		value += ele * weight[0] * tex2D(s1, tex + float2(j, -rcpres.y));
		value += ele * weight[1] * tex2D(s1, tex + float2(j, 0.0));
		value += ele * weight[2] * tex2D(s1, tex + float2(j, rcpres.y));
		value += ele * weight[3] * tex2D(s1, tex + float2(j, rcpres.y * 2.0));
	}		
	return Linear(value / (contrib * contrib));	   
}

Technique T0
{		
	pass p0 {		
		PixelShader = compile ps_2_0 Normalize();
	}	
	pass p1 {
		PixelShader = compile ps_2_a Resample();
	}			
} 